"use strict";
/* eslint-disable @typescript-eslint/no-misused-promises */
/* eslint-disable @typescript-eslint/no-non-null-assertion */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const body_parser_1 = __importDefault(require("body-parser"));
const connect_sdk_1 = require("@shipengine/connect-sdk");
const logger_1 = __importDefault(require("./utils/logger"));
const path_1 = __importDefault(require("path"));
const serialize_error_1 = require("serialize-error");
function logRequest(req, _res, next) {
    logger_1.default.info(`${req.method} ${req.url}`);
    logger_1.default.body(req.body);
    next();
}
function buildImageUrl(pathToApp, pathToImage, port) {
    let fileName = path_1.default.relative(pathToApp, pathToImage);
    const isWin = process.platform === "win32";
    if (isWin) {
        fileName = fileName.replace(/\\/g, '/');
    }
    if (fileName) {
        return `http://localhost:${port}/${fileName}`;
    }
}
function formatError(error) {
    return {
        name: error.name,
        message: error.message,
        code: error.code,
        details: error.details || [],
        originalCode: error.originalCode,
        originalError: serialize_error_1.serializeError(error.originalError),
        stack: error.stack,
    };
}
function buildCarrierAppApi(sdkApp, server) {
    sdkApp.createShipment && server.put("/create-shipment", createShipment);
    sdkApp.cancelShipments && server.put("/cancel-shipments", cancelShipments);
    sdkApp.rateShipment && server.put("/rate-shipment", rateShipment);
    sdkApp.trackShipment && server.put("/track-shipment", trackShipment);
    sdkApp.createManifest && server.put("/create-manifest", createManifest);
    sdkApp.schedulePickup && server.put("/schedule-pickup", schedulePickup);
    sdkApp.cancelPickups && server.put("/cancel-pickups", cancelPickups);
    async function createShipment(req, res) {
        try {
            const { transaction, shipment } = req.body;
            const shipmentResponse = await sdkApp.createShipment(transaction, shipment);
            return res.status(200).send({
                transaction,
                shipmentResponse,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
    async function cancelShipments(req, res) {
        try {
            const { transaction, shipments } = req.body;
            const outcomes = await sdkApp.cancelShipments(transaction, shipments);
            return res.status(200).send({
                transaction,
                outcomes,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
    async function rateShipment(req, res) {
        try {
            const { transaction, shipment } = req.body;
            const rates = await sdkApp.rateShipment(transaction, shipment);
            return res.status(200).send({
                transaction,
                rates,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
    async function trackShipment(req, res) {
        try {
            const { transaction, shipment } = req.body;
            const trackingInfo = await sdkApp.trackShipment(transaction, shipment);
            return res.status(200).send({
                transaction,
                trackingInfo,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
    async function createManifest(req, res) {
        try {
            const { transaction, manifest } = req.body;
            const manifestResponse = await sdkApp.createManifest(transaction, manifest);
            return res.status(200).send({
                transaction,
                manifestResponse,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
    async function schedulePickup(req, res) {
        try {
            const { transaction, pickup } = req.body;
            const pickupResponse = await sdkApp.schedulePickup(transaction, pickup);
            return res.status(200).send({
                transaction,
                pickupResponse,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
    async function cancelPickups(req, res) {
        try {
            const { transaction, pickups } = req.body;
            const pickupsResponse = await sdkApp.cancelPickups(transaction, pickups);
            return res.status(200).send({
                transaction,
                pickupsResponse,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
}
function buildOrderAppApi(sdkApp, server) {
    sdkApp.getSalesOrdersByDate && server.put("/get-sales-orders-by-date", getSalesOrdersByDate);
    sdkApp.shipmentCreated && server.put("/shipment-created", shipmentCreated);
    sdkApp.acknowledgeOrders && server.put("/acknowledge-orders", acknowledgeOrders);
    async function getSalesOrdersByDate(req, res) {
        try {
            const { transaction, range } = req.body;
            const response = await sdkApp.getSalesOrdersByDate(transaction, range);
            return res.status(200).send({
                transaction,
                response,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
    async function shipmentCreated(req, res) {
        try {
            const { transaction, shipment } = req.body;
            const response = await sdkApp.shipmentCreated(transaction, shipment);
            return res.status(200).send({
                transaction,
                response,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
    async function acknowledgeOrders(req, res) {
        try {
            const { transaction, notifications } = req.body;
            const response = await sdkApp.acknowledgeOrders(transaction, notifications);
            return res.status(200).send({
                transaction,
                response,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
}
function buildAPI(pathToApp, sdkApp, server, port) {
    server.use(body_parser_1.default.urlencoded({
        extended: false,
    }));
    server.use(body_parser_1.default.json());
    // We are intentionally not logging this request
    server.get("/", getApp);
    server.use(logRequest);
    sdkApp.connect && server.put("/connect", connect);
    if (sdkApp.type === connect_sdk_1.AppType.Carrier)
        buildCarrierAppApi(sdkApp, server);
    if (sdkApp.type === connect_sdk_1.AppType.Order)
        buildOrderAppApi(sdkApp, server);
    function getApp(_req, res) {
        const sdkAppWithLogos = {
            ...sdkApp,
            ...{
                logo: buildImageUrl(pathToApp, sdkApp.logo, port),
                icon: buildImageUrl(pathToApp, sdkApp.icon, port),
            },
        };
        return res.status(200).json(sdkAppWithLogos);
    }
    async function connect(req, res) {
        try {
            const { transaction, connectionFormData } = req.body;
            await sdkApp.connect(transaction, connectionFormData);
            return res.status(200).send({
                transaction,
            });
        }
        catch (error) {
            return res.status(400).send(formatError(error));
        }
    }
}
exports.default = buildAPI;
//# sourceMappingURL=build-api.js.map